/*
 * Copyright 1993, 2000 Christopher Seiwald.
 *
 * This file is part of Jam - see jam.c for Copyright information.
 */

#include "Parse.hpp"
#include "Jam.hpp"
#include "Scan.hpp"
#include "String.hpp"


using Jam::String;


namespace Jam
{
	Parse* Parse::yypsave;
}

/*
 * parse.c - make and destroy parse trees as driven by the parser
 *
 * 09/07/00 (seiwald) - ref count on PARSE to avoid freeing when used,
 *		as per Matt Armstrong.
 * 09/11/00 (seiwald) - structure reworked to reflect that (*func)()
 *		returns a List *.
 */

namespace Jam
{
	Parse::Parse( 
		List	*(*func)( Parse *p, LOL *args ),
		Parse	*left,
		Parse	*right,
		Parse	*third,
		const char	*string,
		const char	*string1,
		const int	num_ )
		: num(num_)
	{
		this->func = func;
		this->left = left;
		this->right = right;
		this->third = third;
		this->string = string;
		this->string1 = string1;
		this->refs = 1;
	}

	void Parse::AddRef(void)
	{
		++refs;
	}

	void Parse::Release(void)
	{
		if( --refs == 0 )
		{
			delete this;
		}
	}

	void Parse::parse_file( const char *f )
	{
		/* Suspend scan of current file */
		/* and push this new file in the stream */

		yyfparse((char*)f);

		/* Now parse each block of rules and execute it. */
		/* Execute it outside of the parser so that recursive */
		/* calls to yyrun() work (no recursive yyparse's). */

		for(;;)
		{
			LOL l;
			Parse *p;

			/* $(<) and $(>) empty in outer scope. */

			lol_init( &l );

			/* Filled by yyparse() calling parse_save() */

			yypsave = 0;

			/* If parse error or empty parse, outta here */

			if( yyparse() || !( p = yypsave ) )
			break;

			/* Run the parse tree. */

			(*(p->func))( p, &l );

			p->Release();
		}
	}

	void Parse::parse_save( Parse *p )
	{
		yypsave = p;
	}

	Parse::~Parse(void)
	{
		if( string )
		{
			String::freestr( string );
		}

		if( string1 )
		{
			String::freestr( string1 );
		}

		if( left )
		{
			left->Release();
		}

		if( right )
		{
			right->Release();
		}

		if( third )
		{
			third->Release();
		}

		delete this;
	}
}